package org.openswing.swing.pivottable.functions.java;

import java.io.Serializable;

/**
 * <p>Title: OpenSwing Framework</p>
 * <p>Description: Function applied to a data field of a pivot table:
 * it reports the min value in the collection of values that matches the row/column fields criteria.</p>
 * <p>Copyright: Copyright (C) 2006 Mauro Carniel</p>
 *
 * <p> This file is part of OpenSwing Framework.
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the (LGPL) Lesser General Public
 * License as published by the Free Software Foundation;
 *
 *                GNU LESSER GENERAL PUBLIC LICENSE
 *                 Version 2.1, February 1999
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *       The author may be contacted at:
 *           maurocarniel@tin.it</p>
 *
 * @author Mauro Carniel
 * @version 1.0
 */
public class MinFunction extends GenericFunction implements Serializable {

  private Double minValue;


  public MinFunction() {
  }


  public void processValue(Object value) {
    if (minValue==null) {
      if (value!=null && value instanceof Number)
      minValue = new Double( ((Number)value).doubleValue());
    }
    else if (value!=null &&
             value instanceof Number &&
             ((Number)value).doubleValue()<minValue.doubleValue())
      minValue = new Double( ((Number)value).doubleValue() );
  }


  public Double getValue() {
    return minValue;
  }


}
